import { transformError as sdkTransformError } from '@wix/sdk-runtime/transform-error';
import { queryBuilder } from '@wix/sdk-runtime/query-builder';
import { renameKeysFromSDKRequestToRESTRequest, renameKeysFromRESTResponseToSDKResponse, } from '@wix/sdk-runtime/rename-all-nested-keys';
import * as ambassadorWixFunctionsTemplatesV1FunctionTemplate from './functions-templates-v1-function-template-function-templates.http.js';
export var SortOrder;
(function (SortOrder) {
    SortOrder["ASC"] = "ASC";
    SortOrder["DESC"] = "DESC";
})(SortOrder || (SortOrder = {}));
export var WebhookIdentityType;
(function (WebhookIdentityType) {
    WebhookIdentityType["UNKNOWN"] = "UNKNOWN";
    WebhookIdentityType["ANONYMOUS_VISITOR"] = "ANONYMOUS_VISITOR";
    WebhookIdentityType["MEMBER"] = "MEMBER";
    WebhookIdentityType["WIX_USER"] = "WIX_USER";
    WebhookIdentityType["APP"] = "APP";
})(WebhookIdentityType || (WebhookIdentityType = {}));
/**
 * Creates a FunctionTemplate
 * @param functionTemplate - FunctionTemplate to be created.
 * @public
 * @documentationMaturity preview
 * @requiredField functionTemplate
 * @returns The created FunctionTemplate.
 * @fqn wix.functions.templates.v1.FunctionTemplates.CreateFunctionTemplate
 */
export async function createFunctionTemplate(functionTemplate) {
    // @ts-ignore
    const { httpClient, sideEffects } = arguments[1];
    const payload = renameKeysFromSDKRequestToRESTRequest({
        functionTemplate: functionTemplate,
    });
    const reqOpts = ambassadorWixFunctionsTemplatesV1FunctionTemplate.createFunctionTemplate(payload);
    sideEffects?.onSiteCall?.();
    try {
        const result = await httpClient.request(reqOpts);
        sideEffects?.onSuccess?.(result);
        return renameKeysFromRESTResponseToSDKResponse(result.data)
            ?.functionTemplate;
    }
    catch (err) {
        const transformedError = sdkTransformError(err, {
            spreadPathsToArguments: {},
            explicitPathsToArguments: { functionTemplate: '$[0]' },
            singleArgumentUnchanged: false,
        }, ['functionTemplate']);
        sideEffects?.onError?.(err);
        throw transformedError;
    }
}
/**
 * Updates a FunctionTemplate.
 * @param _id - dev center component id
 * @public
 * @documentationMaturity preview
 * @requiredField _id
 * @requiredField functionTemplate
 * @returns Updated FunctionTemplate.
 * @fqn wix.functions.templates.v1.FunctionTemplates.UpdateFunctionTemplate
 */
export async function updateFunctionTemplate(_id, functionTemplate) {
    // @ts-ignore
    const { httpClient, sideEffects } = arguments[2];
    const payload = renameKeysFromSDKRequestToRESTRequest({
        functionTemplate: { ...functionTemplate, id: _id },
    });
    const reqOpts = ambassadorWixFunctionsTemplatesV1FunctionTemplate.updateFunctionTemplate(payload);
    sideEffects?.onSiteCall?.();
    try {
        const result = await httpClient.request(reqOpts);
        sideEffects?.onSuccess?.(result);
        return renameKeysFromRESTResponseToSDKResponse(result.data)
            ?.functionTemplate;
    }
    catch (err) {
        const transformedError = sdkTransformError(err, {
            spreadPathsToArguments: { functionTemplate: '$[1]' },
            explicitPathsToArguments: { 'functionTemplate.id': '$[0]' },
            singleArgumentUnchanged: false,
        }, ['_id', 'functionTemplate']);
        sideEffects?.onError?.(err);
        throw transformedError;
    }
}
/**
 * Retrieves a list of FunctionTemplates, should  we support paging, filtering, and sorting in this api?
 * @public
 * @documentationMaturity preview
 * @permissionScope Manage Functions
 * @permissionScopeId SCOPE.DC-FUNCTIONS.MANAGE-FUNCTIONS
 * @permissionId FUNCTIONS.FUNCTION_TEMPLATE_READ
 * @applicableIdentity APP
 * @fqn wix.functions.templates.v1.FunctionTemplates.QueryFunctionTemplates
 */
export function queryFunctionTemplates() {
    // @ts-ignore
    const { httpClient, sideEffects } = arguments[0];
    return queryBuilder({
        func: async (payload) => {
            const reqOpts = ambassadorWixFunctionsTemplatesV1FunctionTemplate.queryFunctionTemplates(payload);
            sideEffects?.onSiteCall?.();
            try {
                const result = await httpClient.request(reqOpts);
                sideEffects?.onSuccess?.(result);
                return result;
            }
            catch (err) {
                sideEffects?.onError?.(err);
                throw err;
            }
        },
        requestTransformer: (query) => {
            const args = [query, {}];
            return renameKeysFromSDKRequestToRESTRequest({
                ...args?.[1],
                query: args?.[0],
            });
        },
        responseTransformer: ({ data, }) => {
            const transformedData = renameKeysFromRESTResponseToSDKResponse(data);
            return {
                items: transformedData?.functionTemplates,
                pagingMetadata: transformedData?.pagingMetadata,
            };
        },
        errorTransformer: (err) => {
            const transformedError = sdkTransformError(err, {
                spreadPathsToArguments: {},
                explicitPathsToArguments: { query: '$[0]' },
                singleArgumentUnchanged: false,
            });
            throw transformedError;
        },
        pagingMethod: 'CURSOR',
        transformationPaths: {},
    });
}
//# sourceMappingURL=functions-templates-v1-function-template-function-templates.universal.js.map